#!/usr/bin/env python
"""DeadSimpleDeploy

Usage:
deploy <app> [--rev=REV] [--env=ENV] [--conf=CONF]

Options:
--rev=REV           Revision to deploy. [default: master]
--env=ENV           Environment to deploy.
--conf=CONF         Specified config file.
--version           Show version.
-h --help           Show this screen.
"""
from docopt import docopt
import logging
import shutil
import subprocess
import sys
import tempfile

from .common.config import get, parse
from .common.logging import error, info, warn


logger = logging.getLogger(__name__)


def main(args=None):
    if not args:
        args = docopt(__doc__, argv=sys.argv[1:],
                      version='DeadSimpleDeploy v0.0.2')

    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s - %(name)s - %(levelname)s - '
                               '%(message)s')

    app = args.get('<app>')
    rev = args.get('--rev')
    env = args.get('--env')

    conf = args.get('--conf')
    configfile = get(logger, filename=conf)
    config = {
        'after': [],
        'before': [],
        'source': None,
        'target': None,
    }
    parse(logger, configfile, config, app=app, env=env, task='deploy',
          required=['source', 'target'])

    if isinstance(config['before'], str):
        subprocess.Popen(config['before'], shell=True).wait()
    else:
        for script in config['before']:
            subprocess.Popen(script, shell=True).wait()

    tmpdir = tempfile.mkdtemp(prefix='deploy-%s-' % app)

    try:
        init = subprocess.Popen(['git', 'init'], cwd=tmpdir,
                                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        _, err = init.communicate()
        warn(logger, err)
    except OSError:
        error(logger, 'git is not installed or has incorrect permissions.',
              tmpdir=tmpdir)

    sremote = subprocess.Popen(['git', 'remote', 'add', 'source',
                                config['source']], cwd=tmpdir)
    dremote = subprocess.Popen(['git', 'remote', 'add', 'deploy',
                                config['target']], cwd=tmpdir)

    sremote.wait()
    fetch = subprocess.Popen(['git', 'fetch', 'source'], cwd=tmpdir,
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    _, err = fetch.communicate()
    if 'ERROR: Repository not found.' in err:
        error(logger, 'invalid source for %s' % app)

    checkout = subprocess.Popen(['git', 'checkout', rev], cwd=tmpdir,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
    _, err = checkout.communicate()
    if "error: pathspec '%s' did not match" % rev in err:
        error(logger, 'invalid revision %s for app %s' % (rev, app))
    elif "Already on '%s'" % rev in err:
        pass
    else:
        warn(logger, err)

    dremote.wait()
    push = subprocess.Popen(['git', 'push', 'deploy', rev], cwd=tmpdir,
                            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    for line in iter(push.stderr.readline, ''):
        print line.rstrip()
    _, err = push.communicate()
    warn(logger, err)

    if push.returncode:
        error(logger,
              'push %s:%s to %s failed, see output' % (app, rev,
                                                       config['target']))

    if isinstance(config['after'], str):
        subprocess.Popen(config['after'], shell=True).wait()
    else:
        for script in config['after']:
            subprocess.Popen(script, shell=True).wait()

    shutil.rmtree(tmpdir)

    info(logger, 'successfully deployed %s' % app)


if __name__ == '__main__':
    main()
