#!/usr/bin/env python
"""DeadSimpleMonitorRepos

Usage:
monitor-repos <org> [--port=PORT] [--conf=CONF]

Options:
--port=PORT         Host the monitor on a specific port. [default: 5000]
--conf=CONF         Specified config file.
--version           Show version.
-h --help           Show this screen.
"""
from flask import Flask, render_template, request
from flask_sockets import Sockets
from gevent.pywsgi import WSGIServer
from geventwebsocket.handler import WebSocketHandler
import json
import logging
import os
from python_util import docopt
import requests
import sys

from .common.config import get, parse
from .common.logging import warn


TOKEN = os.environ['GITHUB_TOKEN']


config = {
    'base-branch': None,
    'branches': None,
    'org': None,
    'repos': None,
}


webapp = Flask(__name__)
sockets = Sockets(webapp)
logger = logging.getLogger(__name__)


logging.getLogger('geventwebsocket').setLevel(logging.WARNING)
logging.getLogger('requests').setLevel(logging.WARNING)


@sockets.route('/githubWS')
def github_ws(ws):
    while True:
        request_ws = json.loads(ws.receive())

        payload = github(request_ws=request_ws)

        ws.send(payload)


@webapp.route('/github')
def github(request_ws=None):
    headers = {'Authorization': 'token %s' % TOKEN}

    r = request_ws or request.args
    for required in ('repo', 'route'):
        if required not in r:
            warn('Bad request: %s is a required parameter.', required)
            return '?'

    url = 'https://api.github.com/repos/%s/%s/%s' % (config['org'], r['repo'],
                                                     r['route'])
    if 'route_after' in r:
        url += r['route_after']
    if 'state' in r:
        url += '?state=' + r['state']


    response = requests.get(url, headers=headers).content
    data = json.loads(response)

    if 'count' in r:
        return str(len(data))
    if 'lag' in r:
        if 'message' in data:
            # Not tracked
            return '?'

        if data['status'] == 'identical':
            return '='
        elif data['status'] == 'behind':
            return '-' + str(data['behind_by'])
        elif data['status'] == 'ahead':
            return '+' + str(data['ahead_by'])
        else:
            warn('Error: could not parse API response (%x)', data)
            return '?'

    return data


@webapp.route('/')
def dash():
    branches_deployable = filter(lambda x: x != config['base-branch'],
                                 config['branches'])

    return render_template('monitor_repos.html',
                           base_branch=config['base-branch'],
                           branches=config['branches'],
                           branches_deployable=branches_deployable,
                           org=config['org'], repos=config['repos'],
                           thresholds={'issues': 50, 'pulls': 5})


def main(args=None):
    if not args:
        args = docopt(__doc__, argv=sys.argv[1:], transforms={'--port': int},
                      version='DeadSimpleMonitorRepos v0.0.1')

    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s - %(name)s - %(levelname)s - '
                               '%(message)s')

    app = args.get('<org>')

    conf = args.get('--conf')
    configfile = get(logger, filename=conf)
    parse(logger, configfile, config, app=app, task='monitor-repos',
          required=['base-branch', 'branches', 'repos'])

    config['org'] = app # hacky way of accessing this elsewhere

    server = WSGIServer(('0.0.0.0', args.get('--port')), webapp,
                        handler_class=WebSocketHandler)
    server.serve_forever()


if __name__ == '__main__':
    main()
